@@ -1,4 +1,4 @@
-package Mojolicious::Plugin::BootstrapHelpers 0.0176 {
+package Mojolicious::Plugin::BootstrapHelpers 0.0180 {
 
     use strict;
     use warnings;
@@ -261,6 +261,26 @@ Both of these are legal:
          <tr><td>A Table Cell</td></tr>
     %  end
 
+=head3 |link|
+
+All other C<|references|> are also helpers, so C<|link|> needs special mention:
+
+    $linktext, [ $url ], %link_has
+
+=over 4
+
+B<C<$itemtext>>
+
+Mandatory. The text on the link.
+
+B<C<$url>>
+
+Mandatory. It sets the C<href> on the link. L<url_for|Mojolicious::Controller#url_for> is used to create the link.
+
+=back
+
+It is similar to a L<button|/"Buttons">, except that C<$url> is mandatory
+
 
 =head1 HELPERS
 
@@ -400,49 +420,43 @@ A submit button for use in forms. It overrides the build-in submit_button helper
 
 =head3 Syntax
 
-There are two different syntaxes. One for single-button dropdowns and one for multi-button dropdowns.
+There are two different syntaxes. One for single-button groups and one for multi-button groups. The difference is that single-button groups can't change
+anything concerning the buttongroup (e.g. it can't be C<justified>). If you need to do that there is nothing wrong with having a multi-button
+group with just one button.
 
     # multi button
     <%= buttongroup %has,
                     buttons => [
-                        [ |button| ],
-                        {
-                            button => [ |button| ],
-                            items => [
-                                [ $itemtext, [ $url ], %item_has ],
+                        [ |button|,
+                          (items => [
+                                [ |link| ],
                                ($headertext,)
                                ([],)
-                            ]
-                        }
+                           ])
+                        ]
                     ]
     %>
 
     # single button
-    <%= buttongroup {
-                        button => [ |button| ],
-                        items => [
-                            [ $itemtext, [ $url ], %item_has ],
+    <%= buttongroup [ |button|,
+                      (items => [
+                            [ |link| ],
                            ($headertext,)
                            ([],)
-                        ]
-                    }
+                       ])
+                    ]
     %>
 
 B<C<buttons =E<gt> []>>
 
-Single-button: Not available. Multi-button: Mandatory array reference. Takes a list of child elements of two different types:
+The single-button style is a shortcut for the C<buttons> array reference. It takes ordinary L<buttons|/"Buttons">, with two differences: The C<items> array reference, and it is unnecessary to give a button
+with C<items> a url.
 
 =over 4
 
-B<C<[ |button| ]>>
+B<C<items =E<gt> [...]>>
 
-Single-button: Not available. Multi-button: Array references are (and take the same arguments as) ordinary L<buttons|/"Buttons">. Two exceptions: It can't take a url, and it can take the C<caret> strapping.
-
-B<C<{ ... }>>
-
-Hash references are nested L<dropdowns|/"Dropdowns">. Read more there.
-
-For the single-button dropdown, this is the only argument.
+Giving a button an C<items> array reference creates a L<dropdown|/"Dropdowns">. Read more under C<items> there.
 
 =back
 
@@ -732,39 +746,29 @@ A mandatory array reference of L<button groups|/"Button-groups">.
 =head3 Syntax
 
     <%= dropdown  %has,
-                  button => [ |button| ],
-                  items  => [
-                      [ $itemtext, [ $url ], %item_has ],
-                     ($headertext,)
-                     ([],)
+                  [ |button|, items  => [
+                       [ |link| ],
+                      ($headertext,)
+                      ([],)
+                    ]
                   ]
 
-B<C<button =E<gt> []>>
+B<C<[ |button| ]>>
 
-Mandatory array reference. Takes the same arguments as an ordinary L<button|/"Buttons">, with two exceptions: It can't take a url, and it can take the C<caret> strapping.
+Mandatory array reference. It takes an ordinary L<button|/"Buttons">, with two differences: The C<items> array reference, and it is unnecessary to give a button
+with C<items> a url.
+
+=over 4
 
 B<C<items>>
 
-Mandatory array reference. Here are the items that make up the menu. It takes two different types of value (both can occur any number of times:
+Mandatory array reference. Here are the items that make up the menu. It takes three different types of value (both can occur any number of times:
 
 =over 4
 
-B<C<[ $itemtext, [ $url ], %item_has ]>>
-
-This creates a linked menu item.
-
-=over 4
-
-B<C<$itemtext>>
-
-Mandatory. The text on the link.
-
-B<C<$url>>
-
-Mandatory. It sets the C<href> on the link. L<url_for|Mojolicious::Controller#url_for> is used to create the link.
-
-=back
+B<C<[ |link| ]>>
 
+An array reference creates a L<link|/"link"> in the menu.
 
 B<C<$headertext>>
 
@@ -776,6 +780,8 @@ An empty array reference creates a divider.
 
 =back
 
+=back
+
 
 B<Available strappings>
 
@@ -1078,17 +1084,36 @@ Creates a radiobutton by giving its content to L<radio_button|Mojolicious::Plugi
 
 B<C<prepend =E<gt> { buttongroup =E<gt> { |buttongroup| }>>
 
-Creates a single button buttongroup. See L<button_groups|/"Button-groups"> for details.
+Creates a single button buttongroup. See L<button groups|/"Button-groups"> for details.
 
 B<C<prepend =E<gt> { buttongroup =E<gt> [ |buttongroup| ]>>
 
-Creates a multi button buttongroup. See L<button_groups|/"Button-groups"> for details.
+Creates a multi button buttongroup. See L<button groups|/"Button-groups"> for details.
 
 =back
 
 =head3 Examples
 
 
+    <%= input input => { text_field => ['username'] },
+              prepend => { check_box => ['agreed'] }
+    %>
+
+    <div class="input-group">
+        <span class="input-group-addon"><input name="agreed" type="checkbox" /></span>
+        <input class="form-control" id="username" type="text" name="username" />
+    </div>
+
+=begin html
+
+<p>
+An input group with a checkbox.
+
+</p>
+
+=end html
+
+
     <%= input large,
               prepend => { radio_button => ['yes'] },
               input => { text_field => ['username'] },
@@ -1111,6 +1136,455 @@ A <code>large</code> input group with a radio button prepended and a string appe
 =end html
 
 
+    <%= input input => { text_field => ['username'] },
+              append => { button => ['Click me!'] },
+    %>
+
+    <div class="input-group">
+        <input class="form-control" id="username" type="text" name="username" />
+        <span class="input-group-btn"><button class="btn btn-default" type="button">Click me!</button></span>
+    </div>
+
+=begin html
+
+<p>
+An input group with a button.
+
+</p>
+
+=end html
+
+
+
+=begin html
+
+<p>
+    <%= buttongroup ['Default', caret, items  => [
+                        ['Item 1', ['item1'] ],
+                        ['Item 2', ['item2'] ],
+                        [],
+                        ['Item 3', ['item3'] ],
+                    ] ]
+    %>
+</p>
+
+=end html
+
+    <%= input input  => { text_field => ['username'] },
+              append => { buttongroup => [['The button', caret, right, items => [
+                                  ['Item 1', ['item1'] ],
+                                  ['Item 2', ['item2'] ],
+                                  [],
+                                  ['Item 3', ['item3'] ],
+                              ] ] ]
+                        }
+    %>
+
+    <div class="input-group">
+        <input class="form-control" id="username" type="text" name="username" />
+        <div class="input-group-btn">
+            <button class="btn btn-default dropdown-toggle" type="button" data-toggle="dropdown">The button <span class="caret"></span>
+            </button>
+            <ul class="dropdown-menu dropdown-menu-right">
+                <li><a class="menuitem" href="item1" tabindex="-1">Item 1</a></li>
+                <li><a class="menuitem" href="item2" tabindex="-1">Item 2</a></li>
+                <li class="divider"></li>
+                <li><a class="menuitem" href="item3" tabindex="-1">Item 3</a></li>
+            </ul>
+        </div>
+    </div>
+
+=begin html
+
+<p>
+An input group with a button dropdown appended. Note that <code>right</code> is manually applied.
+
+</p>
+
+=end html
+
+
+    <%= input input   => { text_field => ['username'] },
+              prepend => { buttongroup => [
+                              buttons => [
+                                ['Link 1', ['http://www.example.com/'] ],
+                                [undef, caret, items => [
+                                      ['Item 1', ['item1'] ],
+                                      ['Item 2', ['item2'] ],
+                                      [],
+                                      ['Item 3', ['item3'] ],
+                                  ],
+                               ],
+                            ],
+                         ],
+                      },
+    %>
+
+    <div class="input-group">
+        <div class="input-group-btn">
+            <a class="btn btn-default" href="http://www.example.com/">Link 1</a>
+            <div class="btn-group">
+                <button class="btn btn-default dropdown-toggle" type="button" data-toggle="dropdown"><span class="caret"></span>
+                </button>
+                <ul class="dropdown-menu">
+                    <li><a class="menuitem" href="item1" tabindex="-1">Item 1</a></li>
+                    <li><a class="menuitem" href="item2" tabindex="-1">Item 2</a></li>
+                    <li class="divider"></li>
+                    <li><a class="menuitem" href="item3" tabindex="-1">Item 3</a></li>
+                </ul>
+            </div>
+        </div>
+        <input class="form-control" id="username" type="text" name="username" />
+    </div>
+
+=begin html
+
+<p>
+An input group with a split button dropdown prepended.
+</p>
+
+=end html
+
+
+
+
+=head2 Navs
+
+=head3 Syntax
+
+    <%= nav %has,
+            $type => [ |link|,
+                      (items => [
+                            [ |link| ],
+                           ($headertext,)
+                           ([],)
+                       ])
+                    ]
+    %>
+
+C<Navs> are syntactically similar to L<button groups|/"Button-groups">.
+
+B<C<$type =E<gt> [...]>>
+
+Mandatory. C<$type> is either C<pills> or C<tabs> (or C<items> if the C<nav> is in a L<navbar|/"Navbars">) and applies the adequate class to the surrounding C<ul>.
+
+=over 4
+
+B<C<items =E<gt> []>>
+
+If present does the same as C<items> in L<dropdown|/"Dropdowns">.
+
+=back
+
+=head3 Examples
+
+
+    <%= nav pills => [
+                ['Item 1', ['#'] ],
+                ['Item 2', ['#'], active ],
+                ['Item 3', ['#'] ],
+                ['Item 4', ['#'], disabled ],
+            ]
+    %>
+
+    <ul class="nav nav-pills">
+        <li><a href="#">Item 1</a></li>
+        <li class="active"><a href="#">Item 2</a></li>
+        <li><a href="#">Item 3</a></li>
+        <li class="disabled"><a href="#">Item 4</a></li>
+    </ul>
+
+=begin html
+
+<p>
+A simple pills navigation.
+
+</p>
+
+=end html
+
+
+    <%= nav justified, id => 'my-nav', tabs => [
+                ['Item 1', ['#'] ],
+                ['Item 2', ['#'], active ],
+                ['Item 3', ['#'] ],
+                ['Dropdown', ['#'], caret, items => [
+                        ['There are...', ['#'] ],
+                        ['...three...', ['#'] ],
+                        [],
+                        ['...choices', ['#'] ],
+                    ],
+                ],
+            ]
+    %>
+
+    <ul class="nav nav-justified nav-tabs" id="my-nav">
+        <li><a href="#">Item 1</a></li>
+        <li class="active"><a href="#">Item 2</a></li>
+        <li><a href="#">Item 3</a></li>
+        <li class="dropdown">
+            <a class="dropdown-toggle" data-toggle="dropdown" href="#">Dropdown <span class="caret"></span></a>
+            <ul class="dropdown-menu">
+                <li><a href="#">There are...</a></li>
+                <li><a href="#">...three...</a></li>
+                <li class="divider"></li>
+                <li><a href="#">...choices</a></li>
+            </ul>
+        </li>
+    </ul>
+
+=begin html
+
+<p>
+A tab navigation with a menu.
+</p>
+
+=end html
+
+
+
+
+=head2 Navbars
+
+=head3 Syntax
+
+    navbar %has, header => [ |link|, %navbar_has ],
+                 form => [
+                     [ [ $url ], %form_has ],
+                     [
+                         formgroup => [ |formgroup| ],
+                         input => [ |input| ],
+                         button => [ |button| ],
+                         submit_button => [ |submit_button| ],
+                      ]
+                  ],
+                  button => [ |button| ],
+                  nav => [ |nav| ]
+                  p => [ $text, %p_has ]
+
+C<Navbars> are comples structures. They take the following arguments:
+
+B<C<header =E<gt> [ |link|, %navbar_has ]>>
+
+C<header> creates a C<navbar-header>. There can be only one C<header>.
+
+=over 4
+
+B<C<|link|>>
+
+Creates the C<brand>. Set the link text to C<undef> if you don't want a brand.
+
+B<C<%navbar_has>>
+
+Can take the following extra arguments:
+
+=over 4
+
+The C<hamburger> strapping creates the menu button for collapsed navbars.
+
+B<C<toggler =E<gt> $collapse_id>>
+
+This sets the C<id> on the collapsing part of the navbar. Set it if you need to reference that part of the navbar, otherwise an id will be generated.
+
+=back
+
+=back
+
+The following arguments can appear any number of times, and is rendered in order.
+
+=over 4
+
+B<C<button =E<gt> [ |button| ]>>
+
+Creates a L<button|/"Buttons">.
+
+B<C<nav =E<gt> [ |nav| ]>>
+
+Creates a L<nav|/"Navs">. Use C<items> if you need to create submenus.
+
+B<C<p =E<gt> [ $text, %p_has ]>>
+
+Creates a C<E<lt>pE<gt>$textE<lt>/pE<gt>> tag.
+
+B<C<form =E<gt> [...]>>
+
+Creates a C<form>, by leveraging L<form_for|Mojolicious::Plugin::TagHelpers#form_for> in L<Mojolicious::Plugin::TagHelpers>.
+
+=over 4
+
+B<C<[ [ $url ], %form_has ]>>
+
+Mandatory array reference. This sets up the C<form> tag.
+
+B<C<[...]>>
+
+Mandatory array reference. The second argument to C<form> can take different types (any number of times, rendered in order):
+
+=over 4
+
+B<C<formgroup =E<gt> [ |formgroup| ]>>
+B<C<input =E<gt> [ |input| ]>>
+B<C<button =E<gt> [ |button| ]>>
+B<C<submit_button =E<gt> [ |submit_button| ]>>
+
+Creates L<form groups|/"Form-groups">, L<input groups|/"Input-groups">, L<buttons|/"Buttons"> and L<submit_buttons|/"Submit_buttons">
+
+=back
+
+=back
+
+=back
+
+
+    <%= navbar header => ['The brand', ['#'], hamburger, toggler => 'bs-example-navbar-collapse-2'],
+               nav => [ items => [
+                       ['Link', ['#'] ],
+                       ['Another link', ['#'], active ],
+                       ['Menu', ['#'], caret, items => [
+                           ['Choice 1', ['#'] ],
+                           ['Choice 2', ['#'] ],
+                           [],
+                           ['Choice 3', ['#'] ],
+                       ] ],
+                   ]
+               ]
+    %>
+
+    <nav class="navbar navbar-default">
+        <div class="container-fluid">
+            <div class="navbar-header">
+                <button class="collapsed navbar-toggle" data-target="#bs-example-navbar-collapse-2" data-toggle="collapse" type="button">
+                    <span class="icon-bar"></span>
+                    <span class="icon-bar"></span>
+                    <span class="icon-bar"></span>
+                </button>
+                <a class="navbar-brand" href="#">The brand</a>
+            </div>
+            <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-2">
+                <ul class="nav navbar-nav">
+                    <li><a href="#">Link</a></li>
+                    <li class="active"><a href="#">Another link</a></li>
+                    <li class="dropdown">
+                        <a class="dropdown-toggle" data-toggle="dropdown" href="#">Menu <span class="caret"></span></a>
+                        <ul class="dropdown-menu">
+                            <li><a href="#">Choice 1</a></li>
+                            <li><a href="#">Choice 2</a></li>
+                            <li class="divider"></li>
+                            <li><a href="#">Choice 3</a></li>
+                        </ul>
+                    </li>
+                </ul>
+            </div>
+        </div>
+    </nav>
+
+=begin html
+
+<p>
+A simple navbar with a couple of links and a submenu.
+
+</p>
+
+=end html
+
+
+    <%= navbar header => ['Brand', ['#'], hamburger, toggler => 'collapse-4124'],
+               nav => [ items => [
+                       ['Link', ['#'], active ],
+                       ['Link', ['#'] ],
+                       ['Dropdown', ['#'], caret, items => [
+                           ['Action', ['#'] ],
+                           ['Another action', ['#'] ],
+                           ['Something else here', ['#'] ],
+                           [],
+                           ['Separated link', ['#'] ],
+                           [],
+                           ['One more separated link', ['#'] ],
+                       ] ] ],
+                ],
+                form => [
+                    [['/login'], method => 'post', left],
+                    [
+                        formgroup => [
+                            text_field => ['the-search', placeholder => 'Search' ],
+                        ],
+                        submit_button => ['Submit'],
+                    ]
+                ],
+                nav => [
+                    right,
+                    items => [
+                        ['Link', ['#'] ],
+                        ['Dropdown', ['#'], caret, items => [
+                                ['Action', ['#'] ],
+                                ['Another action', ['#'] ],
+                                ['Something else here', ['#'] ],
+                                [],
+                                ['Separated link', ['#'] ],
+                            ],
+                        ]
+                    ],
+                ]
+    %>
+
+    <nav class="navbar navbar-default">
+        <div class="container-fluid">
+            <div class="navbar-header">
+                <button type="button" class="collapsed navbar-toggle" data-toggle="collapse" data-target="#collapse-4124">
+                    <span class="icon-bar"></span>
+                    <span class="icon-bar"></span>
+                    <span class="icon-bar"></span>
+                </button>
+                <a class="navbar-brand" href="#">Brand</a>
+            </div>
+            <div class="collapse navbar-collapse" id="collapse-4124">
+                <ul class="nav navbar-nav">
+                    <li class="active"><a href="#">Link</a></li>
+                    <li><a href="#">Link</a></li>
+                    <li class="dropdown">
+                        <a class="dropdown-toggle" data-toggle="dropdown" href="#">Dropdown <span class="caret"></span></a>
+                        <ul class="dropdown-menu">
+                            <li><a href="#">Action</a></li>
+                            <li><a href="#">Another action</a></li>
+                            <li><a href="#">Something else here</a></li>
+                            <li class="divider"></li>
+                            <li><a href="#">Separated link</a></li>
+                            <li class="divider"></li>
+                            <li><a href="#">One more separated link</a></li>
+                        </ul>
+                    </li>
+                </ul>
+                <form action="/login" class="navbar-form navbar-left" method="post">
+                    <div class="form-group">
+                        <input class="form-control" id="the-search" name="the_search" placeholder="Search" type="text" />
+                    </div>
+                    <button class="btn btn-default" type="submit">Submit</button>
+                </form>
+                <ul class="nav navbar-nav navbar-right">
+                    <li><a href="#">Link</a></li>
+                    <li class="dropdown">
+                        <a class="dropdown-toggle" data-toggle="dropdown" href="#">Dropdown <span class="caret"></span></a>
+                        <ul class="dropdown-menu">
+                            <li><a href="#">Action</a></li>
+                            <li><a href="#">Another action</a></li>
+                            <li><a href="#">Something else here</a></li>
+                            <li class="divider"></li>
+                            <li><a href="#">Separated link</a></li>
+                        </ul>
+                    </li>
+                </ul>
+            </div>
+        </div>
+    </nav>
+
+=begin html
+
+<p>
+This is (almost) identical to the <a href="http://getbootstrap.com/components/#navbar">Bootstrap documentation example</a>.
+</p>
+
+=end html
+
 
 
 =head2 Panels
